from pathlib import Path
from typing import Union

from polylith import configuration, diff, info, repo


def print_views(root: Path, tag: str, short: bool, only_bricks: bool) -> None:
    ns = configuration.get_namespace_from_config(root)
    files = diff.collect.get_files(tag)
    bases = diff.collect.get_changed_bases(files, ns)
    components = diff.collect.get_changed_components(files, ns)

    projects = diff.collect.get_changed_projects(files)
    all_projects_data = info.get_bricks_in_projects(root, components, bases, ns)
    projects_data = [p for p in all_projects_data if info.is_project(p)]

    affected_projects = diff.collect.get_projects_affected_by_changes(
        projects_data, projects, bases, components
    )

    if not short and not only_bricks:
        diff.report.print_diff_summary(tag, bases, components)
        diff.report.print_detected_changes_in_projects(projects, short)
        diff.report.print_diff_details(projects_data, bases, components)

        return

    if short and not only_bricks:
        diff.report.print_projects_affected_by_changes(affected_projects, short)

        return

    diff.report.print_detected_changes_in_bricks(bases, components, short)


def run(tag_name: Union[str, None], short: bool, only_bricks: bool):
    root = repo.get_workspace_root(Path.cwd())

    tag = diff.collect.get_latest_tag(root, tag_name)

    if not tag:
        print("No tags found in repository.")
        return

    print_views(root, tag, short, only_bricks)



from pathlib import Path
from typing import Iterable, List, Union

from poetry.factory import Factory
from poetry.poetry import Poetry
from poetry.utils.env import EnvManager
from polylith import project


def get_project_poetry(poetry: Poetry, path: Union[Path, None]) -> Poetry:
    return Factory().create_poetry(path) if path else poetry


def distributions(poetry: Poetry, path: Union[Path, None]) -> Iterable:
    project_poetry = get_project_poetry(poetry, path)

    env = EnvManager(project_poetry).get()

    return env.site_packages.distributions()


def find_third_party_libs(poetry: Poetry, path: Union[Path, None]) -> dict:
    project_poetry = get_project_poetry(poetry, path)

    if not project_poetry.locker.is_locked():
        raise ValueError("poetry.lock not found. Run `poetry lock` to create it.")

    packages = project_poetry.locker.locked_repository().packages

    return {p.name: str(p.version) for p in packages}


def filter_projects_data(
    poetry: Poetry, directory: Union[str, None], projects_data: List[dict]
) -> List[dict]:
    if not directory:
        return projects_data

    project_name = project.get_project_name(poetry.pyproject.data)

    data = next((p for p in projects_data if p["name"] == project_name), None)

    if not data:
        raise ValueError(f"Didn't find project in {directory}")

    return [data]
